﻿Imports ThinkGeo.MapSuite.DesktopEdition
Imports ThinkGeo.MapSuite.Core
Imports System.Drawing.Drawing2D

Public Class Form1
    Private mobjLayersOverlay As LayerOverlayEX
    Private mFullExtent As RectangleShape
    Private mobjMapUnits As GeographyUnit
    Private mobjProjectionJUR As Proj4Projection

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles Me.Load
        mobjLayersOverlay = New LayerOverlayEX

        LoadBingMapsLayer()

        With mapLink
            .MapUnit = mobjMapUnits
            .Cursor = Cursors.Hand

            .Overlays.Add("LAYERS", mobjLayersOverlay)
            .BackColor = Color.White

            '.ThreadingMode = MapThreadingMode.Multithreaded
            .MapResizeMode = MapResizeMode.PreserveExtent
            .DrawingQuality = DrawingQuality.HighQuality
            .SmoothingMode = SmoothingMode.HighQuality
            .CompositingQuality = CompositingQuality.HighSpeed
            .ExtentOverlay.MouseWheelMode = MapMouseWheelMode.Disabled
            .MaximumScale = Double.MaxValue
            .MinimumScale = 1000   '//(Required) 
            .ZoomLevelSnapping = ZoomLevelSnappingMode.None   '//(Option) 

            '.ExtentOverlay.ZoomPercentage = 50
            .ExtentOverlay.RightClickDragMode = MapRightClickDragMode.Disabled
            .ExtentOverlay.PanMode = MapPanMode.StandardPanning
            .CurrentExtent = mFullExtent
            .Refresh()
        End With
    End Sub

    Private Sub LoadBingMapsLayer()
        Try
            ' Please set your own information about those parameters below.   
            Dim applicationID As String = "YourBingID"
            Dim cacheDirectory As String = Application.StartupPath & "\Bingmaps"

            If Directory.Exists(cacheDirectory) = False Then
                Directory.CreateDirectory(cacheDirectory)
            End If

            Dim objLayer As New BingMapsLayerEX(applicationID, BingMapsMapType.AerialWithLabels, cacheDirectory)
            objLayer.TileCache.ClearCache()

            Dim strKey As String = ""
            objLayer.Logo = Nothing
            objLayer.Name = "Bing Maps"

            strKey = objLayer.Name.Replace(" ", "") & "Layer"

            mobjLayersOverlay.Layers.Add(strKey, objLayer)
            mobjLayersOverlay.Layers.MoveToBottom(strKey)

            mFullExtent = New RectangleShape(-15084380.5844437, 5276411.31850397, -11644096.8502817, 3748285.28774829)
            mobjMapUnits = GeographyUnit.Meter
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    Private Sub mapLink_MouseWheel(sender As Object, e As MouseEventArgs) Handles mapLink.MouseWheel
        'Dim objPoint As PointShape = ExtentHelper.ToWorldCoordinate(mapLink.CurrentExtent, New ScreenPointF(e.X, e.Y), mapLink.Width, mapLink.Height)
        If e.Delta < 0 Then
            ZoomOUT(20) ', New ScreenPointF(e.X, e.Y))
        Else
            ZoomIN(20) ', New ScreenPointF(e.X, e.Y))
        End If
    End Sub

    Private Sub ZoomIN(sngPercent As Single, Optional objPoint As PointShape = Nothing)
        If IsNothing(objPoint) = False Then
            mapLink.ZoomIntoCenter(CInt(sngPercent), objPoint)
        Else
            mapLink.ZoomToScale(mapLink.CurrentScale / (1 + (sngPercent * 0.01)))
        End If
        mapLink.Refresh()
    End Sub

    Private Sub ZoomOUT(sngPercent As Single, Optional objPoint As PointShape = Nothing)
        If IsNothing(objPoint) = False Then
            mapLink.ZoomOutToCenter(CInt(sngPercent), objPoint)
        Else
            mapLink.ZoomToScale(mapLink.CurrentScale * (1 + (sngPercent * 0.01)))
        End If
        mapLink.Refresh()
    End Sub

End Class

Public Class BingMapsLayerEX
    Inherits BingMapsLayer

    Public Sub New(applicationId As String, mapType As BingMapsMapType, cacheDirectory As String)
        MyBase.New(applicationId, mapType, cacheDirectory)
        DrawingExceptionMode = DrawingExceptionMode.DrawException
    End Sub

    Protected Overrides Sub DrawExceptionCore(canvas As GeoCanvas, e As Exception)
        canvas.DrawTextWithScreenCoordinate(e.ToString, New GeoFont("Arial", 14), New GeoSolidBrush(New GeoColor(120, GeoColor.StandardColors.Red)), canvas.Width / 2, canvas.Height / 2, DrawingLevel.LevelOne)
        Clipboard.SetText(e.ToString)
    End Sub
End Class

Public Class LayerOverlayEX
    Inherits LayerOverlay

    Public Sub New()
        DrawingExceptionMode = DrawingExceptionMode.DrawException
    End Sub

    Protected Overrides Sub DrawExceptionCore(canvas As GeoCanvas, e As Exception)
        canvas.DrawTextWithScreenCoordinate(e.ToString, New GeoFont("Arial", 14), New GeoSolidBrush(New GeoColor(120, GeoColor.StandardColors.Red)), canvas.Width / 2, canvas.Height / 2, DrawingLevel.LevelOne)
        Clipboard.SetText(e.ToString)
    End Sub

End Class